﻿//-------------------------------------------------------------
//- Copyright (C) 2017-2018 gamevanilla. All rights reserved. -
//-------------------------------------------------------------

using System.Collections;
using System.Collections.Generic;

using UnityEngine;
using UnityEngine.Assertions;
using UnityEngine.Networking;

namespace CCG
{
    public class Server : NetworkBehaviour
    {
        private List<NetworkConnection> players = new List<NetworkConnection>();

        private int currentPlayerIndex;
        private NetworkConnection currentPlayer;
        private NetworkConnection currentOpponent;

        private const float TurnDuration = 5.0f;

        public override void OnStartServer()
        {
            NetworkServer.RegisterHandler(NetworkProtocol.RegisterPlayerInGame, OnRegisteredPlayerInGame);
        }

        private void OnRegisteredPlayerInGame(NetworkMessage netMsg)
        {
            var msg = netMsg.ReadMessage<NetworkProtocol.RegisterPlayerInGameMessage>();
            Assert.IsNotNull(msg);
            Debug.Log("Player registered into the server.");

            if (!players.Contains(netMsg.conn))
            {
                players.Add(netMsg.conn);
                if (players.Count == 2)
                {
                    Debug.Log("Begin game...");
                    BeginGame();
                }
            }
        }

        private void BeginGame()
        {
            currentPlayerIndex = 0;
            currentPlayer = players[currentPlayerIndex];
            currentOpponent = players[1 - currentPlayerIndex];

            StartCoroutine(BeginGameAsync());
        }

        private void BeginTurn()
        {
            var msg = new NetworkProtocol.BeginTurnMessage();
            msg.ActivePlayer = true;
            NetworkServer.SendToClient(currentPlayer.connectionId, NetworkProtocol.BeginTurn, msg);

            msg.ActivePlayer = false;
            NetworkServer.SendToClient(currentOpponent.connectionId, NetworkProtocol.BeginTurn, msg);
        }

        private void EndTurn()
        {
            var msg = new NetworkProtocol.EndTurnMessage();
            msg.ActivePlayer = true;
            NetworkServer.SendToClient(currentPlayer.connectionId, NetworkProtocol.EndTurn, msg);

            msg.ActivePlayer = false;
            NetworkServer.SendToClient(currentOpponent.connectionId, NetworkProtocol.EndTurn, msg);

            currentPlayerIndex = 1 - currentPlayerIndex;
            currentPlayer = players[currentPlayerIndex];
            currentOpponent = players[1 - currentPlayerIndex];
        }

        private IEnumerator BeginGameAsync()
        {
            while (true)
            {
                BeginTurn();
                yield return new WaitForSeconds(TurnDuration);
                EndTurn();
            }
        }
    }
}
